@@ -22,6 +22,7 @@ import sys
 import logging as log
 from argparse import ArgumentParser, HelpFormatter
 from .config import GlobalConfig
+from .utils import print_error
 from .utils.env import set_unicode_allowed, set_owning_user
 from .osbase import OSBase
 
@@ -48,7 +49,7 @@ def init_config(options):
     if options.owner:
         info = options.owner.split(':')
         if len(info) > 2:
-            print('You can only use one colon to split user:group when using the --owner flag.')
+            print_error('You can only use one colon to split user:group when using the --owner flag.')
             sys.exit(1)
         if len(info) == 2:
             user = info[0]
@@ -72,8 +73,8 @@ def command_create(options):
     ''' Create new container image '''
 
     check_print_version(options)
-    if not options.suite:
-        print('Need at least a suite name to bootstrap!')
+    if not options.name:
+        print_error('Need at least a container name (suite name) to bootstrap!')
         sys.exit(1)
     gconf = init_config(options)
 
@@ -89,7 +90,8 @@ def command_create(options):
                     options.suite,
                     options.arch,
                     variant=options.variant,
-                    base_suite=options.base_suite)
+                    base_suite=options.base_suite,
+                    custom_name=options.name)
     r = osbase.create(options.mirror,
                       components,
                       extra_suites=extra_suites,
@@ -103,11 +105,15 @@ def command_delete(options):
     ''' Delete container image '''
 
     check_print_version(options)
-    if not options.suite:
-        print('No suite name was specified!')
+    if not options.name:
+        print_error('No container image name was specified!')
         sys.exit(1)
     gconf = init_config(options)
-    osbase = OSBase(gconf, options.suite, options.arch, options.variant)
+    osbase = OSBase(gconf,
+                    options.suite,
+                    options.arch,
+                    options.variant,
+                    custom_name=options.name)
     r = osbase.delete()
     if not r:
         sys.exit(2)
@@ -117,11 +123,15 @@ def command_update(options):
     ''' Update container image '''
 
     check_print_version(options)
-    if not options.suite:
-        print('Need at least a suite name for update!')
+    if not options.name:
+        print_error('Need at least a container image name for update!')
         sys.exit(1)
     gconf = init_config(options)
-    osbase = OSBase(gconf, options.suite, options.arch, options.variant)
+    osbase = OSBase(gconf,
+                    options.suite,
+                    options.arch,
+                    options.variant,
+                    custom_name=options.name)
     if options.recreate:
         r = osbase.recreate()
     else:
@@ -146,11 +156,15 @@ def command_build(options):
     from .build import build_from_directory, build_from_dsc
 
     check_print_version(options)
-    if not options.suite:
-        print('Need at least a suite name for building!')
+    if not options.name:
+        print_error('Need at least a container image or suite name for building!')
         sys.exit(1)
     gconf = init_config(options)
-    osbase = OSBase(gconf, options.suite, options.arch, options.variant)
+    osbase = OSBase(gconf,
+                    options.suite,
+                    options.arch,
+                    options.variant,
+                    custom_name=options.name)
 
     # prepare user-defined environment variables
     env_vars = {}
@@ -168,8 +182,8 @@ def command_build(options):
     if options.buildflags:
         buildflags = options.buildflags.split(';')
 
-    if not options.target and os.path.isdir(options.suite):
-        print('A directory is given as parameter, but you are missing a suite parameter to build for.')
+    if not options.target and os.path.isdir(options.name):
+        print('A directory is given as parameter, but you are missing a container-name parameter to build for.')
         print('Can not continue.')
         sys.exit(1)
 
@@ -211,11 +225,15 @@ def command_login(options):
     ''' Open interactive session in a container '''
 
     check_print_version(options)
-    if not options.suite:
-        print('Need at least a suite name!')
+    if not options.name:
+        print_error('Need at least a container image name!')
         sys.exit(1)
     gconf = init_config(options)
-    osbase = OSBase(gconf, options.suite, options.arch, options.variant)
+    osbase = OSBase(gconf,
+                    options.suite,
+                    options.arch,
+                    options.variant,
+                    custom_name=options.name)
 
     allowed = []
     if options.allow:
@@ -229,26 +247,36 @@ def command_run(options, custom_command):
     ''' Run arbitrary command in container session '''
 
     check_print_version(options)
-    if not options.suite:
-        print('Need at least a suite name!')
+    if not options.name:
+        print_error('Need at least a container image name!')
         sys.exit(1)
     gconf = init_config(options)
     osbase = OSBase(gconf,
                     options.suite,
                     options.arch,
                     options.variant,
+                    custom_name=options.name,
                     cachekey=options.cachekey)
 
     allowed = []
     if options.allow:
         allowed = [s.strip() for s in options.allow.split(',')]
 
+    bind_build_dir = options.bind_build_dir
+    if bind_build_dir == 'y':
+        bind_build_dir = 'ro'
+    elif bind_build_dir == 'rw' or bind_build_dir == 'ro':
+        pass
+    else:
+        bind_build_dir = 'n'
+
     r = osbase.run(custom_command,
                    options.build_dir,
                    options.artifacts_dir,
                    init_command=options.init_command,
                    copy_command=options.external_commad,
                    header_msg=options.header,
+                   bind_build_dir=bind_build_dir,
                    allowed=allowed)
     if not r:
         sys.exit(2)
@@ -281,7 +309,7 @@ def command_maintain(options):
         maintain_print_status(gconf)
         return
 
-    print('No maintenance action selected!')
+    print_error('No maintenance action selected!')
     sys.exit(1)
 
 
@@ -295,11 +323,13 @@ class CustomArgparseFormatter(HelpFormatter):
 
 def add_container_select_arguments(parser):
     parser.add_argument('--variant', action='store', dest='variant', default=None,
-                        help='Set the bootstrap script variant.')
+                        help='Set the bootstrap script variant (use `none` to select no variant).')
     parser.add_argument('-a', '--arch', action='store', dest='arch', default=None,
                         help='The architecture of the container.')
-    parser.add_argument('suite', action='store', nargs='?', default=None,
-                        help='The suite name of the container.')
+    parser.add_argument('--suite', action='store', dest='suite', default=None,
+                        help='Explicitly set a suite name (instead of having it derived from the container name).')
+    parser.add_argument('name', action='store', nargs='?', default=None,
+                        help='The name of the container image (usually a distribution suite name).')
 
 
 def create_parser(formatter_class=None):
@@ -367,7 +397,7 @@ def create_parser(formatter_class=None):
     sp = subparsers.add_parser('build', help='Build a package in an isolated environment',
                                formatter_class=formatter_class, aliases=['b'])
     add_container_select_arguments(sp)
-    sp.add_argument('-s', '--sign', action='store_true', dest='sign',
+    sp.add_argument('--sign', action='store_true', dest='sign',
                     help='Sign the resulting package.')
     sp.add_argument('--only', choices=['binary', 'arch', 'indep', 'source'], dest='build_only',
                     help=('CF|Select only a specific set of packages to be built. Choices are:\n'
@@ -418,7 +448,11 @@ def create_parser(formatter_class=None):
     sp.add_argument('--artifacts-out', action='store', dest='artifacts_dir', default=None,
                     help='Directory on the host where artifacts can be stored. Mounted to /srv/artifacts in the guest.')
     sp.add_argument('--build-dir', action='store', dest='build_dir', default=None,
-                    help='Select a host directory that gets bind mounted to /srv/build.')
+                    help='Select a host directory that gets copied to /srv/build.')
+    sp.add_argument('--bind-build-dir', action='store', dest='bind_build_dir', default='n',
+                    choices=['y', 'n', 'ro', 'rw'],
+                    help=('Bindmount build directory instead of copying it. Mounts read-only by default, but can mount '
+                          'as writable as well if \'rw\' is passed as value.'))
     sp.add_argument('--cachekey', action='store', dest='cachekey', default=None,
                     help=('If set, use the specified cache-ID to store an initialized container image for faster '
                           'initialization times.\n'
@@ -458,7 +492,7 @@ def create_parser(formatter_class=None):
 
 def run(mainfile, args):
     if len(args) == 0:
-        print('Need a subcommand to proceed!')
+        print_error('Need a subcommand to proceed!')
         sys.exit(1)
 
     parser = create_parser()
@@ -470,7 +504,7 @@ def run(mainfile, args):
         for i, arg in enumerate(args):
             if arg == '---':
                 if i + 1 == len(args):
-                    print('No command was given after "---", can not continue.')
+                    print_error('No command was given after "---", can not continue.')
                     sys.exit(1)
                 custom_command = args[i + 1:]
                 args = args[:i]
@@ -484,6 +518,6 @@ def run(mainfile, args):
         command_run(args, custom_command)
     else:
         if not hasattr(args, 'func'):
-            print('Unknown or no subcommand was provided. Can not proceed.')
+            print_error('Unknown or no subcommand was provided. Can not proceed.')
             sys.exit(1)
         args.func(args)
