@@ -26,8 +26,8 @@ from contextlib import contextmanager
 from typing import Optional
 from .utils import temp_dir, print_header, print_section, format_filesize, \
     print_info, print_error, print_warn, listify
-from .utils.misc import maybe_remove
-from .utils.env import ensure_root, get_random_free_uid_gid
+from .utils.misc import maybe_remove, safe_copy
+from .utils.env import ensure_root, get_random_free_uid_gid, get_owner_uid_gid
 from .utils.command import safe_run
 from .utils.zstd_tar import compress_directory, decompress_tarball, ensure_tar_zstd
 from .nspawn import nspawn_run_helper_persist, nspawn_run_persist
@@ -51,18 +51,27 @@ class OSBase:
     Describes an OS base registered with debspawn
     '''
 
-    def __init__(self, gconf, suite, arch, variant=None, base_suite=None, cachekey=None):
+    def __init__(self, gconf, suite, arch, variant=None, *,
+                 base_suite=None, custom_name=None, cachekey=None):
         self._gconf = gconf
         self._suite = suite
         self._base_suite = base_suite
         self._arch = arch
-        self._variant = variant
+
+        # if variant is not passed, we use the configured default
+        self._variant = variant if variant else gconf.default_bootstrap_variant
+        if self._variant == 'none':
+            # "none" is an alias to "don't set a variant when invoking debootstrap"
+            self._variant = None
+
+        self._custom_name = custom_name
         self._name = self._make_name()
         self._results_dir = self._gconf.results_dir
         self._cachekey = cachekey
         if self._cachekey:
             self._cachekey = self._cachekey.replace(' ', '')
 
+        self._parameters_checked = False
         self._aptcache = APTCache(self)
 
         # get a fresh UID to give to our build user within the container
@@ -72,17 +81,75 @@ class OSBase:
         ensure_tar_zstd()
 
     def _make_name(self):
+        ''' Configure a unique-ish name based on user defined data,
+        and tweak the custom name and suite values to match. '''
+
+        if self._custom_name and not self._suite:
+            # if we have a custom name but no suite name, the custom name is treated
+            # as our suite name *if* no image exists with the custom name
+            # (this is for backwards compatibility)
+            self._name = self._custom_name
+            if not self.exists():
+                self._suite = self._custom_name
+                self._custom_name = None
+        if self._custom_name == self._suite:
+            self._custom_name = None
+
         if not self._arch:
             out, _, ret = safe_run(['dpkg', '--print-architecture'])
             if ret != 0:
                 raise Exception('Running dpkg --print-architecture failed: {}'.format(out))
 
             self._arch = out.strip()
-        if self._variant:
-            return '{}-{}-{}'.format(self._suite, self._arch, self._variant)
+        if self._custom_name:
+            return self._custom_name
+        elif self._variant:
+            return '{}-{}-{}'.format(self._suite, self._variant, self._arch)
         else:
             return '{}-{}'.format(self._suite, self._arch)
 
+    def _custom_name_parameter_check(self):
+        ''' Read parameters in case a custom name was passed, and perform basic sanity checks. '''
+        import sys
+
+        if self._parameters_checked:
+            return
+        if not self._custom_name:
+            return
+
+        config_fname = self.get_config_location()
+        if not os.path.isfile(config_fname):
+            print_error('No configuration data found for image "{}"!'.format(self.name))
+            sys.exit(3)
+
+        with open(config_fname, 'rt') as f:
+            cdata = json.loads(f.read())
+
+            c_suite = cdata.get('Suite', self.suite)
+            if not self._suite:
+                # if no suite was set, but we have one in the manifest file,
+                # we will always use it to fill in the gap
+                self._suite = c_suite
+
+            if self.suite != c_suite:
+                print_error('Expected suite name "{}" for image "{}", but got "{}" instead.'.format(
+                    cdata.get('Suite'), self.name, self.suite))
+                sys.exit(1)
+            c_arch = cdata.get('Architecture', self.arch)
+            c_variant = cdata.get('Variant', self.variant)
+
+            if self.arch and self.arch != c_arch:
+                print_warn(('Expected architecture "{}" for image "{}", but got "{}" instead. '
+                            'Using expected value.').format(
+                                c_arch, self.name, self.arch))
+            if self.variant and self.variant != c_variant:
+                print_warn(('Expected variant "{}" for image "{}", but got "{}" instead. '
+                            'Using expected value.').format(
+                                c_variant, self.name, self.variant))
+            self._arch = c_arch
+            self._variant = c_variant
+        self._parameters_checked = True
+
     @property
     def name(self) -> str:
         return self._name
@@ -167,10 +234,17 @@ class OSBase:
         program with an error code in case it does not.
         '''
         import sys
-        if not self.exists():
+        if not self._load_existent():
             print_error('The container image for "{}" does not exist. Please create it first.'.format(self.name))
             sys.exit(3)
 
+    def _load_existent(self) -> bool:
+        ''' Check if image exists, and if so load some essential data and return True. '''
+        # ensure the set config values are sane if the user is using a custom container name
+        if self._custom_name:
+            self._custom_name_parameter_check()
+        return self.exists()
+
     def new_nspawn_machine_name(self):
         import platform
         from random import choice
@@ -232,12 +306,37 @@ class OSBase:
         # in the container and may cause issues when bindmounting over it
         maybe_remove(os.path.join(instance_dir, 'etc', 'resolv.conf'))
 
+        # our APT proxy connfiguration should also not be stored, we will reset it every time
+        maybe_remove(os.path.join(instance_dir, 'etc', 'apt', 'apt.conf.d', '98debspawn_proxy'))
+
         # drop machine-id file, if one exists
         maybe_remove(os.path.join(instance_dir, 'etc', 'machine-id'))
         # drop logfiles which we want to reset
         maybe_remove(os.path.join(instance_dir, 'var', 'log', 'lastlog'))
         maybe_remove(os.path.join(instance_dir, 'var', 'log', 'faillog'))
 
+    def _setup_apt_proxy(self, instance_dir):
+        ''' Setup APT proxy configuration.
+        APT needs special treatment even in the container to work with a company proxy (yuck!),
+        so we do this here so the user does not need to care about it.
+        '''
+
+        http_proxy = os.getenv('HTTP_PROXY', os.getenv('http_proxy'))
+        https_proxy = os.getenv('HTTPS_PROXY', os.getenv('https_proxy'))
+        if not http_proxy and not https_proxy:
+            return
+        if http_proxy:
+            http_proxy = http_proxy.replace('"', '')
+        if https_proxy:
+            https_proxy = https_proxy.replace('"', '')
+
+        proxyconf_fname = os.path.join(instance_dir, 'etc', 'apt', 'apt.conf.d', '98debspawn_proxy')
+        with open(proxyconf_fname, 'w') as f:
+            if http_proxy:
+                f.write('Acquire::http::Proxy "{}";\n'.format(http_proxy))
+            if https_proxy:
+                f.write('Acquire::https::Proxy "{}";\n'.format(https_proxy))
+
     def _create_internal(self, mirror=None, components=None,
                          extra_suites: list[str] = None, extra_source_lines: str = None,
                          allow_recommends: bool = False,
@@ -258,6 +357,8 @@ class OSBase:
         else:
             print_section('Creating new base: {} [{}]'.format(self.suite, self.arch))
 
+        if self._custom_name:
+            print('Custom name: {}'.format(self.name))
         print('Using mirror: {}'.format(mirror if mirror else 'default'))
         if self.variant:
             print('variant: {}'.format(self.variant))
@@ -352,6 +453,9 @@ class OSBase:
             # mess with the next step
             self._remove_unwanted_files(tdir)
 
+            # configure APT proxy, so the configure operation will work behind proxys
+            self._setup_apt_proxy(tdir)
+
             print_section('Configure')
             if nspawn_run_helper_persist(self, tdir,
                                          self.new_nspawn_machine_name(),
@@ -397,8 +501,8 @@ class OSBase:
         ''' Remove container base image '''
         ensure_root()
 
-        if not self.exists():
-            print_error('Can not delete "{}": The configuration does not exist.'.format(self.name))
+        if not self._load_existent():
+            print_error('Can not delete "{}": This configuration does not exist.'.format(self.name))
             return False
 
         print_header('Removing base image {}'.format(self.name))
@@ -430,7 +534,10 @@ class OSBase:
                 image_fname = self.get_cache_image_location()
             else:
                 image_fname = self.get_image_location()
+
             decompress_tarball(image_fname, tdir)
+            self._setup_apt_proxy(tdir)
+
             yield tdir, self.new_nspawn_machine_name()
 
     def make_instance_permanent(self, instance_dir):
@@ -461,8 +568,8 @@ class OSBase:
         ''' Update container base image '''
         ensure_root()
 
-        if not self.exists():
-            print_error('Can not update "{}": The configuration does not exist.'.format(self.name))
+        if not self._load_existent():
+            print_error('Can not update "{}": This configuration does not exist.'.format(self.name))
             return False
 
         print_header('Updating container image')
@@ -497,7 +604,7 @@ class OSBase:
         ''' Recreate a container base image '''
         ensure_root()
 
-        if not self.exists():
+        if not self._load_existent():
             print_error('Can not recreate "{}": The image does not exist.'.format(self.name))
             return False
 
@@ -570,8 +677,8 @@ class OSBase:
         ''' Interactive shell login into the container '''
         ensure_root()
 
-        if not self.exists():
-            print_info('Can not enter "{}": The configuration does not exist.'.format(self.name))
+        if not self._load_existent():
+            print_info('Can not enter "{}": This configuration does not exist.'.format(self.name))
             return False
 
         print_header('Login (persistent changes) for {}'.format(self.name)
@@ -597,6 +704,23 @@ class OSBase:
         print_info('Done.')
         return True
 
+    def retrieve_artifacts(self, src_dir: str, dest_dir: Optional[str] = None):
+        from glob import glob
+
+        print_section('Retrieving build artifacts')
+        if not dest_dir:
+            dest_dir = self.results_dir
+
+        o_uid, o_gid = get_owner_uid_gid()
+        acount = 0
+        for f in glob(os.path.join(src_dir, '*.*')):
+            if os.path.isfile(f):
+                target_fname = os.path.join(dest_dir, os.path.basename(f))
+                safe_copy(f, target_fname)
+                os.chown(target_fname, o_uid, o_gid, follow_symlinks=False)
+                acount += 1
+        print_info('Copied {} files.'.format(acount))
+
     def _copy_command_script_to_instance_dir(self, instance_dir: str, command_script: str) -> Optional[str]:
         '''
         Copy a script from the host to the current instance directory and make it
@@ -619,11 +743,11 @@ class OSBase:
         return os.path.join('/srv', 'tmp', os.path.basename(host_script))
 
     def run(self, command, build_dir, artifacts_dir, init_command=None, copy_command=False,
-            header_msg=None, allowed: list[str] = None):
+            header_msg=None, bind_build_dir: Optional[str] = None, allowed: list[str] = None):
         ''' Run an arbitrary command or script in the container '''
         ensure_root()
 
-        if not self.exists():
+        if not self._load_existent():
             print_error('Can not run command in "{}": The base image does not exist.'.format(self.name))
             return False
 
@@ -637,12 +761,14 @@ class OSBase:
                 init_command = shlex.split(init_command)
         init_command = listify(init_command)
         allowed = listify(allowed)
+        if bind_build_dir == 'n':
+            bind_build_dir = None
 
         # ensure we have absolute paths
         if build_dir:
-            build_dir = os.path.abspath(build_dir)
+            build_dir = os.path.normpath(os.path.abspath(build_dir))
         if artifacts_dir:
-            artifacts_dir = os.path.abspath(artifacts_dir)
+            artifacts_dir = os.path.normpath(os.path.abspath(artifacts_dir))
 
         if self._cachekey and init_command and not self.cacheimg_exists():
             print_header('Preparing template for `{}`'.format(self._cachekey))
@@ -652,7 +778,6 @@ class OSBase:
                 # ensure helper script runner exists and is up to date
                 self._copy_helper_script(instance_dir)
 
-                init_nspawn_flags = ['--bind={}:/srv/build/'.format(os.path.normpath(build_dir))]
                 if copy_command:
                     # copy initialization script from host to container
                     host_script = init_command[0]
@@ -678,6 +803,16 @@ class OSBase:
                 for perm in allowed:
                     if perm not in banned_permissions:
                         filtered_allowed.append(perm)
+
+                init_nspawn_flags = []
+                if build_dir:
+                    if not bind_build_dir:
+                        shutil.copytree(build_dir, os.path.join(instance_dir, 'srv', 'build'), dirs_exist_ok=True)
+                    else:
+                        if bind_build_dir == 'rw':
+                            init_nspawn_flags = ['--bind={}:/srv/build/'.format(build_dir)]
+                        elif bind_build_dir == 'ro':
+                            init_nspawn_flags = ['--bind-ro={}:/srv/build/'.format(build_dir)]
                 r = nspawn_run_persist(self,
                                        instance_dir,
                                        machine_name,
@@ -719,15 +854,26 @@ class OSBase:
                 print_error('Container setup failed.')
                 return False
 
-            print_section('Running Task')
+            # Create a few directories we may use for bindmounting if some allow-flags are set,
+            # and which are not commonly present in the base image.
+            # This is only needed for `run` actions, and regular package builds should not require
+            # bindmounts to these directories.
+            os.makedirs(os.path.join(instance_dir, 'lib', 'modules'), exist_ok=True)
+            os.makedirs(os.path.join(instance_dir, 'boot'), exist_ok=True)
+            os.makedirs(os.path.join(instance_dir, 'srv', 'artifacts'), exist_ok=True)
 
+            print_section('Running Task')
             nspawn_flags = []
             chdir = '/srv'
-            if artifacts_dir:
-                nspawn_flags.extend(['--bind={}:/srv/artifacts/'.format(os.path.normpath(artifacts_dir))])
             if build_dir:
-                nspawn_flags.extend(['--bind={}:/srv/build/'.format(os.path.normpath(build_dir))])
                 chdir = '/srv/build'
+                if not bind_build_dir:
+                    shutil.copytree(build_dir, os.path.join(instance_dir, 'srv', 'build'), dirs_exist_ok=True)
+                else:
+                    if bind_build_dir == 'rw':
+                        nspawn_flags.extend(['--bind={}:/srv/build/'.format(build_dir)])
+                    elif bind_build_dir == 'ro':
+                        nspawn_flags.extend(['--bind-ro={}:/srv/build/'.format(build_dir)])
 
             r = nspawn_run_persist(self,
                                    instance_dir,
@@ -739,6 +885,9 @@ class OSBase:
             if r != 0:
                 return False
 
+            # copy results to target directory
+            self.retrieve_artifacts(os.path.join(instance_dir, 'srv', 'artifacts'), artifacts_dir)
+
         print_info('Done.')
         return True
 
